.extended-block-warning{margin-bottom:8px}.extended-block-warning .block-editor-warning__message{width:100%}.extended-block-warning .block-editor-warning__actions{flex-wrap:wrap}.wp-extended-block-wrapper{position:relative}.wp-extended-block-wrapper.is-disabled{opacity:.5}.wp-extended-block-wrapper .videoplayer-play{background-color:transparent;background-image:url(data:image/svg+xml;base64,PHN2ZyB4bWxucz0iaHR0cDovL3d3dy53My5vcmcvMjAwMC9zdmciIHdpZHRoPSIyMiIgaGVpZ2h0PSIyMiIgZmlsbD0ibm9uZSI+PHBhdGggZmlsbD0iI2ZmZiIgZD0iTTYuMjU3IDEuMDc1YS40OC40OCAwIDAgMC0uNTAzLS4wMDcuNTMuNTMgMCAwIDAtLjI1NC40NTh2MTguOTQ4YzAgLjE5LjA5Ny4zNjQuMjU0LjQ1OC4xNTYuMDkzLjM0OS4wOS41MDMtLjAwN2wxNS05LjQ3NEEuNTMuNTMgMCAwIDAgMjEuNSAxMWEuNTMuNTMgMCAwIDAtLjI0My0uNDUxeiIvPjwvc3ZnPg==);background-size:cover;border:none;display:block;height:72px;left:50%;position:absolute;top:50%;transform:translate(-50%,-50%);transition:none;width:72px}.wp-extended-block-wrapper img{display:block;margin:0 auto;max-width:650px;-o-object-fit:cover;object-fit:cover;width:100%}.wp-extended-block-wrapper .extended-block-player__overlay{background-color:transparent;height:100%;left:0;position:absolute;top:0;width:100%;z-index:2}