.ui-datepicker{background-color:#fff;border:1px solid #dcdcde;border-radius:0;border-top:none;box-shadow:0 3px 6px rgba(0,0,0,.075);margin:0;padding:0;width:auto}.ui-datepicker-calendar,.ui-datepicker-header .ui-datepicker-title,.ui-datepicker-header a{color:#444;text-decoration:none}.ui-datepicker *{border-radius:0;padding:0}.ui-datepicker table{border:none;border-collapse:collapse;margin:0;width:auto}.ui-datepicker .ui-datepicker-header,.ui-datepicker .ui-widget-header{background-image:none;border:none;font-weight:400}.ui-datepicker .ui-datepicker-header .ui-state-hover{background:#0000;border-color:#0000;cursor:pointer}.ui-datepicker .ui-datepicker-title{font-size:14px;line-height:14px;margin:0;padding:10px 0;text-align:center}.ui-datepicker .ui-datepicker-next,.ui-datepicker .ui-datepicker-prev{height:34px;position:relative;top:0;width:34px}.ui-datepicker .ui-state-hover.ui-datepicker-next,.ui-datepicker .ui-state-hover.ui-datepicker-prev{border:none}.ui-datepicker .ui-datepicker-prev,.ui-datepicker .ui-datepicker-prev-hover{right:0}.ui-datepicker .ui-datepicker-next,.ui-datepicker .ui-datepicker-next-hover{left:0}.ui-datepicker .ui-datepicker-next span,.ui-datepicker .ui-datepicker-prev span{display:none}.ui-datepicker .ui-datepicker-prev{float:right}.ui-datepicker .ui-datepicker-next{float:left}.ui-datepicker .ui-datepicker-next:before,.ui-datepicker .ui-datepicker-prev:before{font:normal 20px/34px dashicons;padding-right:7px;speak:none;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;height:34px;width:34px}.ui-datepicker .ui-datepicker-prev:before{content:"\f341"}.ui-datepicker .ui-datepicker-next:before{content:"\f345"}.ui-datepicker .ui-datepicker-next-hover:before,.ui-datepicker .ui-datepicker-prev-hover:before{opacity:.7}.ui-datepicker select.ui-datepicker-month,.ui-datepicker select.ui-datepicker-year{width:33%}.ui-datepicker thead{font-weight:600}.ui-datepicker th{border-width:1px;padding:10px}.ui-datepicker td{border:1px solid #f4f4f4;padding:0}.ui-datepicker td.ui-datepicker-other-month{border:#0000}.ui-datepicker td.ui-datepicker-week-end{background-color:#f4f4f4;border:1px solid #f4f4f4}.ui-datepicker td.ui-datepicker-today{background-color:#f0f0c0}.ui-datepicker td.ui-datepicker-current-day{background:#bd8}.ui-datepicker td .ui-state-default{background:#0000;border:none;color:#444;display:block;font-weight:400;padding:5px 10px;text-align:center;text-decoration:none;width:auto}.ui-datepicker td.ui-state-disabled .ui-state-default{opacity:.5}