.backup-storage-space {
	&__help-popover {
		line-height: 28px;
		vertical-align: top;
		display: inline-block;
	}

	&__popover.components-popover {
		.components-popover__content {
			padding: 24px;
			color: var(--jp-gray-70);
			background: #fff;
			border-radius: 2px;
			box-shadow: 0 12px 20px rgba(0, 0, 0, 0.08);
			border: 1px solid #dcdcde;
			text-align: start;
			width: 304px;

			h3 {
				margin-bottom: 16px;
			}

			p {
				font-size: 1rem;
				color: var(--jp-gray-80);
			}

			hr {
				margin-bottom: 0.25rem;
			}

			p > a {
				text-decoration: underline;
			}
		}
	}

	&__toggle-popover {
		&.components-button {
			padding: 0px 4px !important;
			height: auto !important;
			box-shadow: none !important;
			background: transparent;

			.gridicon {
				color: var(--jp-gray-30);
			}
		}

		&.components-button:focus {
			box-shadow: none !important;
			background: transparent;
		}
	}

	&__close-popover {
		position: absolute;
		right: 24px;
		top: 24px;

		&.components-button {
			height: 24px;
			width: 24px;
			padding: 0 !important;
			color: var(--jp-gray-50);
			box-shadow: none !important;

			span {
				height: 18px;
				border: none;
			}
		}

		&.components-button:hover {
			border: 0 !important;
		}
	}

	&__button-section {
		display: flex;
		justify-content: flex-end;
		margin-top: 24px;

		.button {
			padding: 8px 24px;
		}
	}
}
