.add-new-h2,.view-switch,body.no-js #doaction,body.no-js #doaction2,body.no-js .tablenav select[name^=action]{display:none}.column-feedback_from img{float:right;margin-left:10px;margin-top:3px}.widefat .column-feedback_date,.widefat .column-feedback_from,.widefat .column-feedback_source{width:17%}.widefat .column-feedback_response{width:100%}.widefat .column-feedback_response:before{display:none!important}@media screen and (max-width:782px){.widefat .column-feedback_response{padding-right:8px!important}}#export-modal-opener{margin-left:8px}.post-type-feedback .actions{display:inline-flex;margin-bottom:4px}.column-feedback_response .feedback_response__item{display:grid;grid-template-columns:35% 1fr;grid-row-gap:8px}.column-feedback_response .feedback_response__item-key,.column-feedback_response .feedback_response__item-value{align-items:flex-start;display:flex;word-break:break-word}.column-feedback_response .feedback_response__item-value{font-weight:700}.column-feedback_response .feedback_response__mobile-separator{display:block}@media screen and (min-width:783px){.column-feedback_response .feedback_response__mobile-separator{display:none}}.spam a{color:#bc0b0b}.unspam a,.untrash a{color:#d98500}.post-type-feedback #jetpack-check-feedback-spam{margin-top:0}#TB_overlay{opacity:.5}#TB_title{background:none;border:none}#TB_ajaxWindowTitle{display:none}#TB_ajaxContent{display:flex}.feedback-export-modal__wrapper{display:flex;flex-direction:column;padding:0 33px 9px;width:100%}.feedback-export-modal__header{margin-bottom:32px}.feedback-export-modal__header-title{font-size:24px;font-style:normal;font-weight:700;line-height:32px;margin-bottom:8px}#TB_ajaxContent p.feedback-export-modal__header-subtitle{color:#000;font-size:16px;font-style:normal;font-weight:400;letter-spacing:-.02em;line-height:24px;margin-top:8px;padding:0}.feedback-export-modal__content{display:flex;flex-direction:column;flex-grow:1;gap:32px;padding:0}.feedback-export-modal__footer{display:flex;justify-content:space-between;padding:0 16px}.feedback-export-modal__footer-column{align-items:center;display:flex}.feedback-export-modal__footer #jetpack-logo__icon{height:16px;width:16px}.feedback-export-modal__footer #jetpack-logo__icon path{fill:#000}.feedback-export-modal__footer .feedback-export-modal__footer-link{color:#23282d;font-size:12px;font-weight:600;letter-spacing:-.02em;line-height:0;padding-right:8px;text-align:right;text-decoration:none}.export-card{border-radius:var(--jp-border-radius,4px);box-shadow:0 0 0 1px var(--jp-gray-10,#c3c4c7) inset;display:flex;flex-direction:column;isolation:isolate;padding:24px}.export-card__body,.export-card__header{display:flex}.export-card__header{align-items:center;gap:16px;margin-bottom:16px}.export-card__header-title{font-size:20px;font-style:normal;font-weight:700;line-height:30px}.export-card__body{align-items:center;gap:32px;justify-content:space-between}.export-card__body-description{color:#000;flex-grow:1;font-size:14px;font-style:normal;font-weight:400;letter-spacing:-.02em;line-height:24px;max-width:60%}.export-card__body-description a{color:inherit}.export-card__body-description-footer{color:#757575;font-size:12px;font-style:normal;font-weight:400;line-height:150%;margin-bottom:0}.export-card__body-cta{align-self:flex-end}.export-card__body-cta .button-primary.export-button{background:#000;border:1px solid #fff;border-radius:4px;color:#fff;font-style:normal;font-weight:400;letter-spacing:-.01em}.export-card__beta-badge{background:#2fb41f;border-radius:2px;color:#fff;font-size:11px;font-style:normal;font-weight:600;line-height:150%;padding:4px 8px;text-align:left;text-transform:uppercase}