i.mce-i-grunion{font-family:dashicons;font-size:20px}.jetpack-contact-form-icon:before,i.mce-i-grunion:before{content:"\f175";vertical-align:top}.jetpack-contact-form-icon{display:inline-block;font:18px/1 dashicons;height:18px;opacity:.7;vertical-align:text-top}