@import '@automattic/jetpack-base-styles/style';

.backup-storage-space__meta {
	display: flex;
	justify-content: space-between;

	@include for-phone-down {
		flex-direction: column;
		gap: 4px;
	}

	div, a {
		color: var(--jp-gray-80);
		font-weight: 400;
		line-height: 20px;
	}

	.backup-storage-space__usage-text {
		font-size: 16px;
	}

	.backup-storage-space__retention a {
		font-size: 14px;
	}
}