@mixin clear-text-icon {
	-webkit-font-smoothing: antialiased;
	-moz-osx-font-smoothing: grayscale;
	display: inline-block;
	vertical-align: middle;
}

@mixin dashicons {
	@include clear-text-icon;
	font: normal 16px/1 'Dashicons';
}

@mixin noticons {
	@include clear-text-icon;
	font: normal 16px/1 'Noticons';
}