.a8c-faux-inline-help{bottom:24px;position:fixed;right:24px;z-index:9999}.a8c-faux-inline-help .a8c-faux-inline-help__button{background-color:#1d2327;background-color:var(--color-primary);border:1px solid #3c434a;border:1px solid var(--color-primary-dark);border-radius:100%;bottom:10px;box-shadow:0 4px 5px 0 rgba(16,21,23,.14),0 1px 10px 0 rgba(16,21,23,.12),0 2px 4px -1px rgba(16,21,23,.2);box-shadow:0 4px 5px 0 rgba(var(--color-neutral-100-rgb),.14),0 1px 10px 0 rgba(var(--color-neutral-100-rgb),.12),0 2px 4px -1px rgba(var(--color-neutral-100-rgb),.2);box-sizing:border-box;height:40px;line-height:0;overflow:visible;padding:1px;position:absolute;right:0;transition:all .2s ease-in-out;width:40px}.a8c-faux-inline-help .a8c-faux-inline-help__button:before{background:#fff;background:var(--color-surface);border-radius:100%;content:"";display:block;height:28px;left:5px;position:absolute;top:5px;width:28px}.a8c-faux-inline-help .a8c-faux-inline-help__button:focus{background-color:#1d2327;background-color:var(--color-primary);box-shadow:0 0 0 2px #8c8f94;box-shadow:0 0 0 2px var(--color-primary-light)}.a8c-faux-inline-help .a8c-faux-inline-help__button .gridicon{pointer-events:none;position:relative;fill:#1d2327;fill:var(--color-primary);height:42px;margin:-3px 0 0 -3px;top:0;width:42px}.a8c-faux-inline-help .a8c-faux-inline-help__button .gridicon>g:first-child,.a8c-faux-inline-help .a8c-faux-inline-help__button .gridicon>use:first-child{transform:none}.a8c-faux-inline-help .a8c-faux-inline-help__button:hover:not(.is-active){background:#1d2327;background:var(--color-primary);transform:scale(1.15)}