@import '@automattic/jetpack-base-styles/style';
@import '../masthead/calypso-mixins';

.jp-header,
.jp-footer {
	padding: 20px 0;

	@include for-tablet-up {
		padding: 40px 0;
	}
}

.jp-content {
	position: relative;
	font-size: var(--font-body);
	line-height: 1.5;

	h1,
	h2,
	h3,
	h4,
	h5,
	h6 {
		margin-top: 0;
		line-height: 1.2;
	}

	h1 {
		font-size: var(--font-title-large);
		font-weight: 600;
	}

	h2 {
		font-size: var(--font-title-small);
		font-weight: 500;
	}

	.jp-section {
		h2,
		h3 {
			margin-bottom: 16px;
		}

		p {
			margin-top: 16px;
		}
	}

	p,
	li {
		font-size: 16px;
		line-height: 1.5;
	}

	.jp-connection-status-card h3,
	.jpb-my-plan-container h3 {
		margin-top: 48px;
		font-size: var(--font-title-small);
		font-weight: 500;
	}

	a {
		color: var(--jp-black);
		transition: color, background-color 0.15s ease-out;

		&:hover {
			text-decoration-thickness: var(--jp-underline-thickness);
		}

		&:focus {
			outline-color: var(--jp-black);
		}

		svg.components-external-link__icon {
			margin-left: 4px;
		}
	}

	.button {
		display: inline-block;
		padding: 8px 24px;
		font-weight: 500;
		font-size: 16px;
		color: var(--jp-white);
		background: var(--jp-black);
		text-decoration: none;
		border-radius: var(--jp-border-radius);
		border: 0;
		border-color: var(--jp-black);

		&.is-full-width {
			width: 100%;
			text-align: center;
		}

		&:hover,
		&:active {
			background: var(--jp-black-80);
			color: var(--jp-white);
		}

		&:focus {
			box-shadow: 0 0 0 1px var(--jp-white) inset, 0 0 0 2px var(--jp-black);
		}

		&:disabled,
		&.disabled {
			background: var(--jp-gray);
			pointer-events: none;
		}
	}

	ul.jp-product-promote li {
		background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAADAAAAAwCAMAAABg3Am1AAAANlBMVEVHcEwFnwUInggGnggGnggHnAcAnwUFnQcAnwcGnwkFnQgGnQgFnwcGnQYFnQcFnAcGnQkDnwdhiL0pAAAAEnRSTlMAMF//f2Aw7yBQ3+9gcIBgcED+HDbkAAAAZklEQVR4Ae3LNwICARDDQC0+cv7/Y8mwV9odSfWIcf/+VegnGkIvDaGXKvTTn/Gz+Uf5xTL0K1XotS7fs5H6GHvvaO8d7c3j7rdgHne/A/PYt/cO+R42oYdN6OEQetiFHo4A//6dAXqtBEkmtWutAAAAAElFTkSuQmCC)
			no-repeat;
		background-size: 24px;
		padding-left: 30px;
		margin-bottom: 9px;
		color: var(--jp-black);
	}
}

.jp-hero {
	margin-bottom: 64px;
	padding: 64px 0;
	background: var(--jp-white-off);

	&.is-backup-performing {
		background-image: url("data:image/svg+xml,%3Csvg width='624' height='400' fill='none' xmlns='http://www.w3.org/2000/svg'%3E%3Cpath fill-rule='evenodd' clip-rule='evenodd' d='M624 88.387C575.321 34.082 504.674 0 426 0 323.405 0 234.3 58.22 189.925 143.42 83.07 154.78 0 245.305 0 355c0 38.16 10.072 73.999 27.698 105H624V88.387z' fill='%23fff'/%3E%3C/svg%3E");
		background-position: 100% 100%;
		background-repeat: no-repeat;
	}

	&.is-backup-healthy,
	&.is-backup-error {
		display: none;
	}

	p + .button {
		margin-top: 18px;
	}
}

.jetpack-backup-jitm-card {
	.jitm-card {
		margin-right: 0;
	}
}

.jp-dashboard-footer {
	padding: 40px 0;
}

.jp-connection-status-card--status {
	margin: 30px 0;
}

.jp-connection__connect-screen-layout__left {
	h2 {
		font-weight: bold;
		font-size: 36px;
		margin-top: 32px;
	}

	h3 {
		margin-top: 32px;
	}
}

.jetpack-connection-verified-error {
	margin: 25px 0;
}

.jetpack-admin-page__header {
	display: flex;
	justify-content: space-between;
	align-items: center;
	flex-wrap: wrap;
	gap: calc(var(--horizontal-spacing) * 3);

	.jetpack-admin-page__logo {
		flex-shrink: 0;
	}
}
