// Add percentage of white to a color
// Copyright © 2011–2015 thoughtbot. See CREDITS.md#L3
@function tint($color, $percent){
  @return mix(white, $color, $percent);
}

// Add percentage of black to a color
// Copyright © 2011–2015 thoughtbot. See CREDITS.md#L3
@function shade($color, $percent){
  @return mix(black, $color, $percent);
}
