#adminmenu .toplevel_page_site-notices .upsell_banner{background-color:#fff;color:#000}#adminmenu .toplevel_page_site-notices .upsell_banner .button{background-color:#c9356e;color:#fff}#adminmenu .toplevel_page_site-notices .upsell_banner .button:hover{background-color:#ab235a}