(()=>{function t(t,e){document.querySelectorAll(t).forEach((t=>{for(const[o,c]of Object.entries(e))t.style[o]=c}))}function e(t,e,o){document.querySelectorAll(t).forEach((t=>{!1===e?(t.style.clip="rect(1px, 1px, 1px, 1px)",t.style.height="1px",t.style.overflow="hidden",t.style.position="absolute",t.style.width="1px",document.body.classList.add(o)):(t.style.clip="auto",t.style.height="auto",t.style.overflow="visible",t.style.position="relative",t.style.width="auto",document.body.classList.remove(o))}))}wp.customize("jetpack_content_blog_display",(function(e){function o(e){const o=".jetpack-blog-display.jetpack-the-content",c=".jetpack-blog-display.jetpack-the-excerpt",n=".featured-content .jetpack-blog-display";if("content"===e?(t(`${c}, ${n}.jetpack-the-excerpt`,{clip:"rect(1px, 1px, 1px, 1px)",position:"absolute"}),t(`${o}, ${n}.jetpack-the-content`,{clip:"auto",position:"relative"})):"excerpt"===e?(t(`${o}, ${n}.jetpack-the-content`,{clip:"rect(1px, 1px, 1px, 1px)",position:"absolute"}),t(`${c}, ${n}.jetpack-the-excerpt`,{clip:"auto",position:"relative"})):"mixed"===e&&(t(`${o}.output-the-content, ${n}.jetpack-the-content.output-the-content`,{clip:"auto",position:"relative"}),t(`${c}.output-the-content, ${o}.output-the-excerpt, ${n}.jetpack-the-excerpt.output-the-content, ${n}.jetpack-the-content.output-the-excerpt`,{clip:"rect(1px, 1px, 1px, 1px)",position:"absolute"}),t(`${c}.output-the-excerpt, ${n}.jetpack-the-excerpt.output-the-excerpt`,{clip:"auto",position:"relative"})),blogDisplay.masonry){const t=document.querySelector(blogDisplay.masonry);t&&t.masonry()}}o(blogDisplay.display),e.bind(o)})),wp.customize("jetpack_content_post_details_date",(function(t){t.bind((function(t){e(postDetails.date,t,"date-hidden")}))})),wp.customize("jetpack_content_post_details_categories",(function(t){t.bind((function(t){e(postDetails.categories,t,"categories-hidden")}))})),wp.customize("jetpack_content_post_details_tags",(function(t){t.bind((function(t){e(postDetails.tags,t,"tags-hidden")}))})),wp.customize("jetpack_content_post_details_author",(function(t){t.bind((function(t){e(postDetails.author,t,"author-hidden")}))})),wp.customize("jetpack_content_post_details_comment",(function(t){t.bind((function(t){e(postDetails.comment,t,"comment-hidden")}))}))})();