.posts-page .info{display:inline-block;margin-right:8px;position:relative}.posts-page .info .message{background-color:#fff;border:1px solid #c3c4c7;border-radius:2px;box-shadow:0 2px 5px rgba(0,0,0,.1),0 0 56px rgba(0,0,0,.075);color:#646970;font-size:.875rem;margin-right:5px;padding:16px;position:absolute;text-align:right;visibility:hidden;width:220px;z-index:1}.posts-page .info:hover .message{visibility:visible}@media screen and (max-width:782px){.posts-page .info .icon{font-size:28px;height:28px;width:28px}}