.jp-backup-why-i-need-vp {
    margin-top: 1.5rem;
    padding: calc(var(--spacing-base) * 3);

    color: var(--jp-black);
    background-color: var(--jp-white);
    border-radius: var(--jp-border-radius);
    box-shadow: 0 0 40px rgba(0, 0, 0, 0.08);

    // Need full classname here for specificity purposes
    .jp-backup-why-i-need-vp__title {
        margin-bottom: 3rem;

        font-size: var(--font-title-large);
        font-weight: 700;
    }

    &__reasons {
        display: grid;
        grid-template-columns: 1fr;
        gap: 3rem;
    }

    &__reason_section {
        display: flex;
        flex-direction: column;
        align-items: flex-start;
        gap: 0.5rem;

        max-width: 450px;

        img {
            margin-bottom: 0.5rem;
        }

        span {
            font-size: var(--font-title-medium);
            font-weight: 500;
            line-height: 1.2;
        }

        p {
            margin: 0;

            line-height: 1.5;
        }
    }

    @media (min-width: 600px) {
        padding: calc(var(--spacing-base) * 8);
    }

    @media (min-width: 900px) {
        &__reasons {
            grid-template-columns: repeat(2, 1fr);
        }
    }
}