#adminmenu .toplevel_page_site-notices .upsell_banner{background-color:#2c3338;color:#fff}#adminmenu .toplevel_page_site-notices .upsell_banner .button{background-color:#044b7a;color:#fff}#adminmenu .toplevel_page_site-notices .upsell_banner .button:hover{background-color:#055d9c}