<?php //phpcs:ignore Squiz.Commenting.FileComment.Missing
/**
 * Theme Tools: Jetpack Color.
 *
 * @package automattic/jetpack-classic-theme-helper
 */

require_once __DIR__ . '/../_inc/lib/class.color.php';
