<?php
// i18n-map.php @generated by automattic/jetpack-composer-plugin
return array(
  'domain' => 'jetpack',
  'type' => 'plugins',
  'packages' => array(
    'jetpack-admin-ui' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-admin-ui',
      'ver' => '0.5.2',
    ),
    'jetpack-assets' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-assets',
      'ver' => '4.0.4',
    ),
    'jetpack-backup-pkg' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-backup',
      'ver' => '4.0.8',
    ),
    'jetpack-blaze' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-blaze',
      'ver' => '0.25.7',
    ),
    'jetpack-boost-core' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-boost-core',
      'ver' => '0.3.5',
    ),
    'jetpack-boost-speed-score' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-boost-speed-score',
      'ver' => '0.4.1',
    ),
    'jetpack-classic-theme-helper' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-classic-theme-helper',
      'ver' => '0.9.3',
    ),
    'jetpack-compat' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-compat',
      'ver' => '4.0.0',
    ),
    'jetpack-config' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-config',
      'ver' => '3.0.0',
    ),
    'jetpack-connection' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-connection',
      'ver' => '6.3.2',
    ),
    'jetpack-explat' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-explat',
      'ver' => '0.2.5',
    ),
    'jetpack-forms' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-forms',
      'ver' => '0.36.0',
    ),
    'jetpack-image-cdn' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-image-cdn',
      'ver' => '0.7.4',
    ),
    'jetpack-import' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-import',
      'ver' => '0.9.3',
    ),
    'jetpack-ip' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-ip',
      'ver' => '0.4.1',
    ),
    'jetpack-jitm' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-jitm',
      'ver' => '4.0.5',
    ),
    'jetpack-licensing' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-licensing',
      'ver' => '3.0.4',
    ),
    'jetpack-masterbar' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-masterbar',
      'ver' => '0.12.1',
    ),
    'jetpack-my-jetpack' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-my-jetpack',
      'ver' => '5.4.1',
    ),
    'jetpack-password-checker' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-password-checker',
      'ver' => '0.4.2',
    ),
    'jetpack-plugins-installer' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-plugins-installer',
      'ver' => '0.5.0',
    ),
    'jetpack-post-list' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-post-list',
      'ver' => '0.8.0',
    ),
    'jetpack-protect-models' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-protect-models',
      'ver' => '0.4.2',
    ),
    'jetpack-protect-status' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-protect-status',
      'ver' => '0.4.3',
    ),
    'jetpack-publicize-pkg' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-publicize',
      'ver' => '0.59.0',
    ),
    'jetpack-search-pkg' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-search',
      'ver' => '0.47.8',
    ),
    'jetpack-stats' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-stats',
      'ver' => '0.15.1',
    ),
    'jetpack-stats-admin' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-stats-admin',
      'ver' => '0.24.1',
    ),
    'jetpack-sync' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-sync',
      'ver' => '4.6.0',
    ),
    'jetpack-videopress-pkg' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-videopress',
      'ver' => '0.25.9',
    ),
    'jetpack-waf' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-waf',
      'ver' => '0.23.3',
    ),
    'jetpack-wordads' => array(
      'path' => 'jetpack_vendor/automattic/jetpack-wordads',
      'ver' => '0.4.7',
    ),
    'woocommerce-analytics' => array(
      'path' => 'jetpack_vendor/automattic/woocommerce-analytics',
      'ver' => '0.4.1',
    ),
  ),
);
