.stat-block {
	display: flex;
	flex-direction: column;
	flex: 1;

	.icon {
		img {
			margin-top: 24px;
			height: 24px;
			width: 24px;
		}
		flex: 2;
	}

	.label {
		flex: 1;
		font-weight: bold;
	}

	.value {
		flex: 1;
	}

	.large-text {
		font-size: 2em;
		font-weight: bold;
	}
}

.backup__card {
	padding: 24px;
	background:var( --jp-white );
	box-shadow: 0px 0px 40px rgba( 0, 0, 0, 0.08 );
	border-radius: var( --jp-border-radius );
}

.backup__card-details-items {
	margin: 16px 0 -8px;
	font-weight: 500;
}

.backup__card-details-amount {
	margin-bottom: -12px;
	font-size: var( --font-title-large );
	font-weight: 600;
}