.block-editor-block-list__layout .block-editor-block-list__block.has-warning.is-interactive>*{pointer-events:auto;user-select:auto}.block-editor-block-list__layout .block-editor-block-list__block.has-warning.is-interactive:after{content:none}.jetpack-editor-panel-logo{margin-right:.5em}.blaze-panel .blaze-panel-outbound-link__external_icon{height:1.4em;margin-right:.1em;vertical-align:middle;width:1.4em;fill:currentColor}.blaze-panel .components-panel__body-toggle svg{margin:0 .5em 0 0}.blaze-panel .post-publish-panel__postpublish-buttons .components-button{padding-bottom:.4em;padding-top:.5em}.blaze-panel p{margin-top:0}