export const SITE_BACKUP_POLICIES_GET = 'SITE_BACKUP_POLICIES_GET';
export const SITE_BACKUP_POLICIES_GET_FAILED = 'SITE_BACKUP_POLICIES_GET_FAILED';
export const SITE_BACKUP_POLICIES_GET_SUCCESS = 'SITE_BACKUP_POLICIES_GET_SUCCESS';
export const SITE_BACKUP_SIZE_GET = 'SITE_BACKUP_SIZE_GET';
export const SITE_BACKUP_SIZE_GET_FAILED = 'SITE_BACKUP_SIZE_GET_FAILED';
export const SITE_BACKUP_SIZE_GET_SUCCESS = 'SITE_BACKUP_SIZE_GET_SUCCESS';
export const SITE_BACKUP_STORAGE_ADDON_OFFER_SET = 'SITE_BACKUP_STORAGE_ADDON_OFFER_SET';
export const SITE_BACKUP_STORAGE_SET = 'SITE_BACKUP_STORAGE_SET';
export const SITE_BACKUPS_GET = 'SITE_BACKUPS_GET';
export const SITE_BACKUPS_GET_FAILED = 'SITE_BACKUPS_GET_FAILED';
export const SITE_BACKUPS_GET_SUCCESS = 'SITE_BACKUPS_GET_SUCCESS';
