<?php
/**
 * Array that contains all the core dashicons.
 *
 * @package automattic/jetpack-masterbar
 */

return array(
	'dashicons-admin-appearance'          => true,
	'dashicons-admin-collapse'            => true,
	'dashicons-admin-comments'            => true,
	'dashicons-admin-customizer'          => true,
	'dashicons-admin-generic'             => true,
	'dashicons-admin-home'                => true,
	'dashicons-admin-links'               => true,
	'dashicons-admin-media'               => true,
	'dashicons-admin-multisite'           => true,
	'dashicons-admin-network'             => true,
	'dashicons-admin-page'                => true,
	'dashicons-admin-plugins'             => true,
	'dashicons-admin-post'                => true,
	'dashicons-admin-settings'            => true,
	'dashicons-admin-site-alt'            => true,
	'dashicons-admin-site-alt2'           => true,
	'dashicons-admin-site-alt3'           => true,
	'dashicons-admin-site'                => true,
	'dashicons-admin-tools'               => true,
	'dashicons-admin-users'               => true,
	'dashicons-airplane'                  => true,
	'dashicons-album'                     => true,
	'dashicons-align-center'              => true,
	'dashicons-align-full-width'          => true,
	'dashicons-align-left'                => true,
	'dashicons-align-none'                => true,
	'dashicons-align-pull-left'           => true,
	'dashicons-align-pull-right'          => true,
	'dashicons-align-right'               => true,
	'dashicons-align-wide'                => true,
	'dashicons-amazon'                    => true,
	'dashicons-analytics'                 => true,
	'dashicons-archive'                   => true,
	'dashicons-arrow-down-alt'            => true,
	'dashicons-arrow-down-alt2'           => true,
	'dashicons-arrow-down'                => true,
	'dashicons-arrow-left-alt'            => true,
	'dashicons-arrow-left-alt2'           => true,
	'dashicons-arrow-left'                => true,
	'dashicons-arrow-right-alt'           => true,
	'dashicons-arrow-right-alt2'          => true,
	'dashicons-arrow-right'               => true,
	'dashicons-arrow-up-alt'              => true,
	'dashicons-arrow-up-alt2'             => true,
	'dashicons-arrow-up-duplicate'        => true,
	'dashicons-arrow-up'                  => true,
	'dashicons-art'                       => true,
	'dashicons-awards'                    => true,
	'dashicons-backup'                    => true,
	'dashicons-bank'                      => true,
	'dashicons-beer'                      => true,
	'dashicons-bell'                      => true,
	'dashicons-block-default'             => true,
	'dashicons-book-alt'                  => true,
	'dashicons-book'                      => true,
	'dashicons-buddicons-activity'        => true,
	'dashicons-buddicons-bbpress-logo'    => true,
	'dashicons-buddicons-buddypress-logo' => true,
	'dashicons-buddicons-community'       => true,
	'dashicons-buddicons-forums'          => true,
	'dashicons-buddicons-friends'         => true,
	'dashicons-buddicons-groups'          => true,
	'dashicons-buddicons-pm'              => true,
	'dashicons-buddicons-replies'         => true,
	'dashicons-buddicons-topics'          => true,
	'dashicons-buddicons-tracking'        => true,
	'dashicons-building'                  => true,
	'dashicons-businessman'               => true,
	'dashicons-businessperson'            => true,
	'dashicons-businesswoman'             => true,
	'dashicons-button'                    => true,
	'dashicons-calculator'                => true,
	'dashicons-calendar-alt'              => true,
	'dashicons-calendar'                  => true,
	'dashicons-camera-alt'                => true,
	'dashicons-camera'                    => true,
	'dashicons-car'                       => true,
	'dashicons-carrot'                    => true,
	'dashicons-cart'                      => true,
	'dashicons-category'                  => true,
	'dashicons-chart-area'                => true,
	'dashicons-chart-bar'                 => true,
	'dashicons-chart-line'                => true,
	'dashicons-chart-pie'                 => true,
	'dashicons-clipboard'                 => true,
	'dashicons-clock'                     => true,
	'dashicons-cloud-saved'               => true,
	'dashicons-cloud-upload'              => true,
	'dashicons-cloud'                     => true,
	'dashicons-code-standards'            => true,
	'dashicons-coffee'                    => true,
	'dashicons-color-picker'              => true,
	'dashicons-columns'                   => true,
	'dashicons-controls-back'             => true,
	'dashicons-controls-forward'          => true,
	'dashicons-controls-pause'            => true,
	'dashicons-controls-play'             => true,
	'dashicons-controls-repeat'           => true,
	'dashicons-controls-skipback'         => true,
	'dashicons-controls-skipforward'      => true,
	'dashicons-controls-volumeoff'        => true,
	'dashicons-controls-volumeon'         => true,
	'dashicons-cover-image'               => true,
	'dashicons-dashboard'                 => true,
	'dashicons-database-add'              => true,
	'dashicons-database-export'           => true,
	'dashicons-database-import'           => true,
	'dashicons-database-remove'           => true,
	'dashicons-database-view'             => true,
	'dashicons-database'                  => true,
	'dashicons-desktop'                   => true,
	'dashicons-dismiss'                   => true,
	'dashicons-download'                  => true,
	'dashicons-drumstick'                 => true,
	'dashicons-edit-large'                => true,
	'dashicons-edit-page'                 => true,
	'dashicons-edit'                      => true,
	'dashicons-editor-aligncenter'        => true,
	'dashicons-editor-alignleft'          => true,
	'dashicons-editor-alignright'         => true,
	'dashicons-editor-bold'               => true,
	'dashicons-editor-break'              => true,
	'dashicons-editor-code-duplicate'     => true,
	'dashicons-editor-code'               => true,
	'dashicons-editor-contract'           => true,
	'dashicons-editor-customchar'         => true,
	'dashicons-editor-expand'             => true,
	'dashicons-editor-help'               => true,
	'dashicons-editor-indent'             => true,
	'dashicons-editor-insertmore'         => true,
	'dashicons-editor-italic'             => true,
	'dashicons-editor-justify'            => true,
	'dashicons-editor-kitchensink'        => true,
	'dashicons-editor-ltr'                => true,
	'dashicons-editor-ol-rtl'             => true,
	'dashicons-editor-ol'                 => true,
	'dashicons-editor-outdent'            => true,
	'dashicons-editor-paragraph'          => true,
	'dashicons-editor-paste-text'         => true,
	'dashicons-editor-paste-word'         => true,
	'dashicons-editor-quote'              => true,
	'dashicons-editor-removeformatting'   => true,
	'dashicons-editor-rtl'                => true,
	'dashicons-editor-spellcheck'         => true,
	'dashicons-editor-strikethrough'      => true,
	'dashicons-editor-table'              => true,
	'dashicons-editor-textcolor'          => true,
	'dashicons-editor-ul'                 => true,
	'dashicons-editor-underline'          => true,
	'dashicons-editor-unlink'             => true,
	'dashicons-editor-video'              => true,
	'dashicons-ellipsis'                  => true,
	'dashicons-email-alt'                 => true,
	'dashicons-email-alt2'                => true,
	'dashicons-email'                     => true,
	'dashicons-embed-audio'               => true,
	'dashicons-embed-generic'             => true,
	'dashicons-embed-photo'               => true,
	'dashicons-embed-post'                => true,
	'dashicons-embed-video'               => true,
	'dashicons-excerpt-view'              => true,
	'dashicons-exit'                      => true,
	'dashicons-external'                  => true,
	'dashicons-facebook-alt'              => true,
	'dashicons-facebook'                  => true,
	'dashicons-feedback'                  => true,
	'dashicons-filter'                    => true,
	'dashicons-flag'                      => true,
	'dashicons-food'                      => true,
	'dashicons-format-aside'              => true,
	'dashicons-format-audio'              => true,
	'dashicons-format-chat'               => true,
	'dashicons-format-gallery'            => true,
	'dashicons-format-image'              => true,
	'dashicons-format-quote'              => true,
	'dashicons-format-status'             => true,
	'dashicons-format-video'              => true,
	'dashicons-forms'                     => true,
	'dashicons-fullscreen-alt'            => true,
	'dashicons-fullscreen-exit-alt'       => true,
	'dashicons-games'                     => true,
	'dashicons-google'                    => true,
	'dashicons-googleplus'                => true,
	'dashicons-grid-view'                 => true,
	'dashicons-groups'                    => true,
	'dashicons-hammer'                    => true,
	'dashicons-heading'                   => true,
	'dashicons-heart'                     => true,
	'dashicons-hidden'                    => true,
	'dashicons-hourglass'                 => true,
	'dashicons-html'                      => true,
	'dashicons-id-alt'                    => true,
	'dashicons-id'                        => true,
	'dashicons-image-crop'                => true,
	'dashicons-image-filter'              => true,
	'dashicons-image-flip-horizontal'     => true,
	'dashicons-image-flip-vertical'       => true,
	'dashicons-image-rotate-left'         => true,
	'dashicons-image-rotate-right'        => true,
	'dashicons-image-rotate'              => true,
	'dashicons-images-alt'                => true,
	'dashicons-images-alt2'               => true,
	'dashicons-index-card'                => true,
	'dashicons-info-outline'              => true,
	'dashicons-info'                      => true,
	'dashicons-insert-after'              => true,
	'dashicons-insert-before'             => true,
	'dashicons-insert'                    => true,
	'dashicons-instagram'                 => true,
	'dashicons-laptop'                    => true,
	'dashicons-layout'                    => true,
	'dashicons-leftright'                 => true,
	'dashicons-lightbulb'                 => true,
	'dashicons-linkedin'                  => true,
	'dashicons-list-view'                 => true,
	'dashicons-location-alt'              => true,
	'dashicons-location'                  => true,
	'dashicons-lock-duplicate'            => true,
	'dashicons-lock'                      => true,
	'dashicons-marker'                    => true,
	'dashicons-media-archive'             => true,
	'dashicons-media-audio'               => true,
	'dashicons-media-code'                => true,
	'dashicons-media-default'             => true,
	'dashicons-media-document'            => true,
	'dashicons-media-interactive'         => true,
	'dashicons-media-spreadsheet'         => true,
	'dashicons-media-text'                => true,
	'dashicons-media-video'               => true,
	'dashicons-megaphone'                 => true,
	'dashicons-menu-alt'                  => true,
	'dashicons-menu-alt2'                 => true,
	'dashicons-menu-alt3'                 => true,
	'dashicons-menu'                      => true,
	'dashicons-microphone'                => true,
	'dashicons-migrate'                   => true,
	'dashicons-minus'                     => true,
	'dashicons-money-alt'                 => true,
	'dashicons-money'                     => true,
	'dashicons-move'                      => true,
	'dashicons-nametag'                   => true,
	'dashicons-networking'                => true,
	'dashicons-no-alt'                    => true,
	'dashicons-no'                        => true,
	'dashicons-open-folder'               => true,
	'dashicons-palmtree'                  => true,
	'dashicons-paperclip'                 => true,
	'dashicons-pdf'                       => true,
	'dashicons-performance'               => true,
	'dashicons-pets'                      => true,
	'dashicons-phone'                     => true,
	'dashicons-pinterest'                 => true,
	'dashicons-playlist-audio'            => true,
	'dashicons-playlist-video'            => true,
	'dashicons-plugins-checked'           => true,
	'dashicons-plus-alt'                  => true,
	'dashicons-plus-alt2'                 => true,
	'dashicons-plus'                      => true,
	'dashicons-podio'                     => true,
	'dashicons-portfolio'                 => true,
	'dashicons-post-status'               => true,
	'dashicons-pressthis'                 => true,
	'dashicons-printer'                   => true,
	'dashicons-privacy'                   => true,
	'dashicons-products'                  => true,
	'dashicons-randomize'                 => true,
	'dashicons-reddit'                    => true,
	'dashicons-redo'                      => true,
	'dashicons-remove'                    => true,
	'dashicons-rest-api'                  => true,
	'dashicons-rss'                       => true,
	'dashicons-saved'                     => true,
	'dashicons-schedule'                  => true,
	'dashicons-screenoptions'             => true,
	'dashicons-search'                    => true,
	'dashicons-share-alt'                 => true,
	'dashicons-share-alt2'                => true,
	'dashicons-share'                     => true,
	'dashicons-shield-alt'                => true,
	'dashicons-shield'                    => true,
	'dashicons-shortcode'                 => true,
	'dashicons-slides'                    => true,
	'dashicons-smartphone'                => true,
	'dashicons-smiley'                    => true,
	'dashicons-sort'                      => true,
	'dashicons-sos'                       => true,
	'dashicons-spotify'                   => true,
	'dashicons-star-empty'                => true,
	'dashicons-star-filled'               => true,
	'dashicons-star-half'                 => true,
	'dashicons-sticky'                    => true,
	'dashicons-store'                     => true,
	'dashicons-superhero-alt'             => true,
	'dashicons-superhero'                 => true,
	'dashicons-table-col-after'           => true,
	'dashicons-table-col-before'          => true,
	'dashicons-table-col-delete'          => true,
	'dashicons-table-row-after'           => true,
	'dashicons-table-row-before'          => true,
	'dashicons-table-row-delete'          => true,
	'dashicons-tablet'                    => true,
	'dashicons-tag'                       => true,
	'dashicons-tagcloud'                  => true,
	'dashicons-testimonial'               => true,
	'dashicons-text-page'                 => true,
	'dashicons-text'                      => true,
	'dashicons-thumbs-down'               => true,
	'dashicons-thumbs-up'                 => true,
	'dashicons-tickets-alt'               => true,
	'dashicons-tickets'                   => true,
	'dashicons-tide'                      => true,
	'dashicons-translation'               => true,
	'dashicons-trash'                     => true,
	'dashicons-twitch'                    => true,
	'dashicons-twitter-alt'               => true,
	'dashicons-twitter'                   => true,
	'dashicons-undo'                      => true,
	'dashicons-universal-access-alt'      => true,
	'dashicons-universal-access'          => true,
	'dashicons-unlock'                    => true,
	'dashicons-update-alt'                => true,
	'dashicons-update'                    => true,
	'dashicons-upload'                    => true,
	'dashicons-vault'                     => true,
	'dashicons-video-alt'                 => true,
	'dashicons-video-alt2'                => true,
	'dashicons-video-alt3'                => true,
	'dashicons-visibility'                => true,
	'dashicons-warning'                   => true,
	'dashicons-welcome-add-page'          => true,
	'dashicons-welcome-comments'          => true,
	'dashicons-welcome-learn-more'        => true,
	'dashicons-welcome-view-site'         => true,
	'dashicons-welcome-widgets-menus'     => true,
	'dashicons-welcome-write-blog'        => true,
	'dashicons-whatsapp'                  => true,
	'dashicons-wordpress-alt'             => true,
	'dashicons-wordpress'                 => true,
	'dashicons-xing'                      => true,
	'dashicons-yes-alt'                   => true,
	'dashicons-yes'                       => true,
	'dashicons-youtube'                   => true,
	'dashicons-editor-distractionfree'    => true,
	'dashicons-exerpt-view'               => true,
	'dashicons-format-links'              => true,
	'dashicons-format-standard'           => true,
	'dashicons-post-trash'                => true,
	'dashicons-share1'                    => true,
	'dashicons-welcome-edit-page'         => true,
);
